<?php
/**
 * Slider customizer control.
 *
 * @since 4.2.0
 * @package WC_BOGOF
 */

defined( 'ABSPATH' ) || exit;

/**
 * WC_BOGOF_Customizer class.
 */
class WC_BOGOF_Customize_Control_Slider extends WC_BOGOF_Customize_Control {

	/**
	 * The control type.
	 *
	 * @var string
	 */
	public $type = 'wc-bogof-slider';

	/**
	 * Unit text.
	 *
	 * @var string
	 */
	protected $unit = '';

	/**
	 * Render the control's content.
	 */
	protected function render_content() {
		static::ouput(
			[
				'id'          => '_customize-input-' . $this->id,
				'label'       => $this->label,
				'value'       => $this->value(),
				'description' => $this->description,
				'unit'        => $this->unit,
				'input_attrs' => $this->input_attrs,
				'link'        => $this->get_link(),
			]
		);
	}

	/**
	 * Output the slider template.
	 *
	 * @param array $args Array of agrs.
	 */
	public static function ouput( $args ) {
		$args = (object) wp_parse_args(
			$args,
			[
				'id'          => '',
				'label'       => false,
				'description' => false,
				'value'       => '',
				'unit'        => '',
				'input_attrs' => [],
				'link'        => '',
			]
		);

		if ( empty( $args->input_attrs['id'] ) ) {
			$args->input_attrs['id'] = $args->id;
		}

		$args->input_attrs['value'] = $args->value;
		$args->input_attrs['type']  = 'number';
		?>
		<div class="wc-bogof-slider-control">
			<div class="wc-bogof-control-label -has-tools">
				<?php if ( ! empty( $args->label ) ) : ?>
				<label class="customize-control-title"><?php echo esc_html( $args->label ); ?></label>
				<?php endif; ?>
				<div class="wc-bogof-label-tools">
				<?php if ( ! empty( $args->unit ) ) : ?>
					<span class="wc-bogof-tools-unit">
						<?php echo esc_html( $args->unit ); ?>
					</span>
				<?php endif; ?>
				<?php if ( $args->value ) : ?>
					<button data-reset-input="<?php echo esc_attr( $args->input_attrs['id'] ); ?>" title="<?php echo esc_html_e( 'Reset', 'wc-buy-one-get-one-free' ); ?>" class="components-button wc-bogof-reset-control" disabled="disabled">
						<span class="dashicons dashicons-image-rotate"></span>
					</button>
				<?php endif; ?>
				</div>
			</div>
			<div class="wc-bogof-slider-area">
				<div class="wc-bogof-slider"></div>
				<div class="wc-bogof-slider-input-wrapper<?php echo ( empty( $args->unit ) ? '' : ' -has-unit' ); ?>">
					<input <?php echo wp_kses_post( wc_implode_html_attributes( $args->input_attrs ) . ' ' . $args->link ); ?> />
				</div>
			</div>
			<?php if ( ! empty( $args->description ) ) : ?>
			<p class="description customize-control-description">
				<?php echo esc_html( $args->description ); ?>
			</p>
			<?php endif; ?>
		</div>
		<?php
	}
}
